// core functions for MinimizeToTray

if (!('extensions' in window))
    window.extensions = new Array();

if (!('mook' in window.extensions))
    window.extensions.mook = new Array();

if (!('minimizetotray' in window.extensions.mook))
    window.extensions.mook.minimizetotray = 
    function(){
        // this is a stub only
        throw Components.results.NS_ERROR_ABORT;
    };

///// constants
    window.extensions.mook.minimizetotray
.k_contractid_windowWatcher =
    '@minimizetotray.mozdev.org/minimizetotray/window-watcher;1';

    window.extensions.mook.minimizetotray
.k_contractid_windowHider =
    '@minimizetotray.mozdev.org/minimizetotray/window-hider;1';

    window.extensions.mook.minimizetotray
.k_pref_prefix =
    'extensions.minimizetotray.';

    window.extensions.mook.minimizetotray
.k_xul_prefix =
    'extensions.mook.minimizetotray.';

///// component handles
    window.extensions.mook.minimizetotray
.m_windowWatcher = null;

    window.extensions.mook.minimizetotray
.m_windowHider = null;

    window.extensions.mook.minimizetotray
.m_prefs = null;


///// global variables
    window.extensions.mook.minimizetotray
.m_isTurboMode = false;

/////methods
    window.extensions.mook.minimizetotray
.init = 
function () {
    // because |this| is |window|, where the event fired,
    // not |window.extensions.mook.minimizetotray|
    var self = window.extensions.mook.minimizetotray;
    
    // check to be sure that both app-neutral and app-specific parts
    // of the script has been loaded
    if (!('m_bLoadedCore' in self) || (!self.m_bLoadedCore) ||
        !('m_bLoadedApp' in self) || (!self.m_bLoadedApp)) {
        window.setTimeout(
            self.init, 10);
        return;
    }
    
    // this needs to be triggered from a new event
    // due to bug 174320
    window.setTimeout(
        function() {
            window.removeEventListener(
                "load", 
                window.extensions.mook.minimizetotray.init, true);
        }, 
        0);
    
    // we get an extra fire because the remove listener hasn't fired
    // so we just ignore the inconsistency - pretend it never happened
    if (self.m_windowWatcher)
        return;
        //throw Components.results.NS_ERROR_ALREADY_INITIALIZED;
    
    self.m_prefs =
        Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefBranch);
    
    
    self.noBinaryComponentWarning();
    
    self.m_windowWatcher =
        Components.classes[self.k_contractid_windowWatcher]
            .createInstance(Components.interfaces.trayIWindowWatcher);
    self.m_windowWatcher.init(
        self.getDocShell(window),
        self.windowWatcherCallback);
    
    self.turboModeTest();
};

    window.extensions.mook.minimizetotray
.windowWatcherCallback = 
function( aEvent, aParam, aKeyMask, aData1, aData2) {
    // note that |this| is the function itself
    var self = window.extensions.mook.minimizetotray;
    var iface = Components.interfaces.trayIWindowCallback;
    var rv = false;
    switch (aEvent) {
        case iface.EVENT_WINDOW_MINIMIZE:
            // window is trying to minimize normally
            if (!self.m_prefs.getBoolPref(
                self.k_pref_prefix + 'always'))
                break;
            if (aKeyMask & iface.KEY_CONTROL)
                self.minimizeAll();
            else
                self.minimizeWindow();
            rv = true;
            break;
        case iface.EVENT_WINDOW_ACTIVATE:
            // this window is being forcibly shown
            self.restore();
            break;
        case iface.EVENT_MOUSEDOWN_RIGHT:
            if (iface.PARAM_MOUSE_MINIMIZE == aParam) {
                if (aKeyMask & iface.KEY_CONTROL)
                    self.minimizeAll();
                else
                    self.minimizeWindow();
                rv = true;
            }
            break;
        case iface.EVENT_MOUSEDOWN_MIDDLE:
            if (iface.PARAM_MOUSE_MINIMIZE == aParam) {
                self.minimizeAll();
                rv = true;
            }
            break;
    }
    return rv;
};

    window.extensions.mook.minimizetotray
.minimizeWindow = 
function() {
    this.noBinaryComponentWarning();
    this.restore();
    this.m_windowHider =
        Components.classes[this.k_contractid_windowHider]
            .createInstance(Components.interfaces.trayIWindowHider);
    var docShells = [this.getDocShell(window)];
    this.m_windowHider.minimize(
        docShells.length, 
        docShells,
        this.trayCallback);
};

    window.extensions.mook.minimizetotray
.minimizeAll = 
function() {
    this.noBinaryComponentWarning();
    this.restore();
    this.m_windowHider =
        Components.classes[this.k_contractid_windowHider]
            .createInstance(Components.interfaces.trayIWindowHider);
    var docShells = new Array();
    var wm = 
        Components.classes["@mozilla.org/appshell/window-mediator;1"]
            .getService(Components.interfaces.nsIWindowMediator);
    var e = wm.getEnumerator(null);
    while (e.hasMoreElements()) {
        var domWindow = e.getNext();
        docShells[docShells.length] = this.getDocShell(domWindow);
    } 
    this.m_windowHider.minimize(
        docShells.length, 
        docShells,
        this.trayCallback);
};

    window.extensions.mook.minimizetotray
.restore =
function( ) {
    // need to remove the handle to the windowHider first, because
    // we activate the window when it is restored, thus firing a new
    // JS restore() while we're in the middle of the call to
    // trayIWindowHider::Restore()
    var wh = this.m_windowHider;
    this.m_windowHider = null;
    if (wh)
        wh.restore();
}

    window.extensions.mook.minimizetotray
.getDocShell = 
function( win ) {
    var rv;
    try
    {
        var requestor = 
            win.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
        var nav = 
            requestor.getInterface(Components.interfaces.nsIWebNavigation);
        var dsti = 
            nav.QueryInterface(Components.interfaces.nsIDocShellTreeItem);
        var owner = dsti.treeOwner;
        requestor = 
            owner.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
        rv = requestor.getInterface(Components.interfaces.nsIXULWindow);
        rv = rv.docShell;
        rv = rv.QueryInterface(Components.interfaces.nsIDocShell);
    }
    catch (ex)
    {
        rv = null;
        /* ignore no-interface exception */
    }
    return rv;    
};

    window.extensions.mook.minimizetotray
.noBinaryComponentWarning = 
function( ) {
    if (!(
        this.k_contractid_windowWatcher in Components.classes &&
        this.k_contractid_windowHider in Components.classes
        )) {
        // can't find binary component
        var promptService =
            Components.classes['@mozilla.org/embedcomp/prompt-service;1']
                .getService(Components.interfaces.nsIPromptService);
        promptService.alert(
            window,
            "Minimize To Tray - Component Load Error",
            'Cannot find binary component:\n' +
            'Please make sure you have the required DLLs ' +
            'before installing the extension');
        // prevent popping up two warnings
        this.m_windowWatcher = true;
        throw Components.results.NS_ERROR_NOT_AVAILABLE;
    }
};

    window.extensions.mook.minimizetotray
.turboModeTest = function ( ) {
    var commandLine =
        Components.classes['@mozilla.org/appshell/commandLineService;1']
            .getService(Components.interfaces.nsICmdLineService);
    if (commandLine.getCmdLineValue('-turbo')) {
        // run in -turbo mode (quicklaunch)
        window.setTimeout(
            this.turboMode,
            100);
    }
};

    window.extensions.mook.minimizetotray
.showPopup = function ( popup, x, y ) {

    // adjust popup position once we know width / height
    popup._minimizetotray_onshown = function(event) {
        var popup = event.target;
        
        // prevent infinite recursion (trips bug 174320)
        popup.removeEventListener(
            "popupshown",
            popup._minimizetotray_onshown,
            true);
        
        var box = popup.popupBoxObject;
        var x = popup.targetX;
        var y = popup.targetY;
        if (x + box.width > screen.width)
            x = popup.targetX - box.width;
        if (y + box.height > screen.height)
            y = popup.targetY - box.height;
        
        // re-show the popup in the right position
        popup.hidePopup();
        document.popupNode = null;
        popup.showPopup(
            document.documentElement,
            x, y,
            "context",
            "", "");
    }
    
    popup.addEventListener(
        "popupshown",
        popup._minimizetotray_onshown,
        true);
    
    popup.targetX = x;
    popup.targetY = y;
    popup.showPopup(
        document.documentElement,   // anchoring element
        -999999,                    // x
        -999999,                    // y
        "context",                  // type
        "",                         // popupanchor (ignored)
        "");                        // popupalign (ignored)
};

window.addEventListener(
    "load", 
    window.extensions.mook.minimizetotray.init, 
    true
);

// signal that the core has loaded
window.extensions.mook.minimizetotray.m_bLoadedCore = true;
